
<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/aspplus/include/header.inc" -->

<h4>DataGrid</h4>

<div class="indent" style="width:660">
These samples illustrate using the DataGrid Web control.  These examples use sample data rather than data from a real
database.  Please see the <a href="/quickstart/aspplus/doc/webdataaccess.aspx" target="content">Server-Side Data Access</a> section for
examples of DataGrid bound to live data.
</div>

<p>
<div class="indent" style="font-family:Verdana; font-size:8pt;">
    <b> <img align="middle" src="/quickstart/images/bullet.gif">&nbsp;&nbsp;</b>
    <a class="toc2" target="content" href="#working">Working With DataGrid</a><br>
    <b> <img align="middle" src="/quickstart/images/bullet.gif">&nbsp;&nbsp;</b>
    <a class="toc2" target="content" href="#columns">Defining Columns in DataGrid</a><br>
    <b> <img align="middle" src="/quickstart/images/bullet.gif">&nbsp;&nbsp;</b>
    <a class="toc2" target="content" href="#editing">Editing Data in DataGrid</a><br>
    <b> <img align="middle" src="/quickstart/images/bullet.gif">&nbsp;&nbsp;</b>
    <a class="toc2" target="content" href="#hiding">Hiding Columns in DataGrid</a><br>
    <b> <img align="middle" src="/quickstart/images/bullet.gif">&nbsp;&nbsp;</b>
    <a class="toc2" target="content" href="#sorting">Sorting Columns in DataGrid</a><br>
    <b> <img align="middle" src="/quickstart/images/bullet.gif">&nbsp;&nbsp;</b>
    <a class="toc2" target="content" href="#paging">Paging in DataGrid</a><br>
</div>

<p>
<hr>

<!--BEGIN SECTION-->
<br>
<a name="working">
<span class="subhead">Working With DataGrid</span>
<p>The DataGrid server control displays tabular data and optionally supports selecting, sorting, paging and 
editing the data.  By default, DataGrid generates a bound column for each field in the data source (<b>AutoGenerateColumns</b>=true).
Each field in the data is rendered in a separate column, in the order it occurs in the data.  Field names appear in the grid's
column headers, and values are rendered in text labels.  A default format is applied to non-string values.

<p>This sample illustrates using a simple DataGrid control.

<p>
<!--basic-->
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid1.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid1.src"
  Icon="/quickstart/aspplus/images/ref_datagrid1.gif"
  Caption="DataGrid1.aspx"
  runat="server" />


<!--BEGIN SECTION-->
<br>
<a name="columns">
<span class="subhead">Defining Columns in DataGrid</span>
<p>You can control the order, behavior and rendering of individual columns by directly manipulating the grid's 
<b>Columns</b> collection.  The standard column type -- bound column -- renders the values in text labels.  
The grid also supports other column types that render differently.  Any of the column types may be used together 
with the columns collection of a DataGrid.<p>

<p>Note that you may use explicitly-declared columns together with auto-generated columns (AutoGenerateColumns=true).  
When used together, the explicitly-declared columns in the Columns collection are rendered first, then the 
auto-generated columns are rendered.  The auto-generated columns are not added to the Columns collection.</p>

<div class="indent">
<table class="table2" width="75%">
    <tr>
    <th style="width:25%"><b>Column Name</b></th>
    <th>Description</th>
    </tr>
    <tr>
    <td style="width:25%"><b>Bound Column</b></td>
    <td>Lets you control the order and rendering of the columns.</td>
    </tr>
    <tr>
    <td style="width:25%"><b>HyperLink Column</b></td>
    <td>Presents the bound data in HyperLink controls.</td>
    </tr>
    <tr>
    <td style="width:25%"><b>Button Column</b></td>
    <td>Bubbles a user command from within a row to an event handler on the Grid.</td>
    </tr>
    <tr>
    <td style="width:25%"><b>Template Column</b></td>
    <td>Lets you control which controls are rendered in the column.</td>
    </tr>
    <td style="width:25%"><b>Edit Command Column</b></td>
    <td>Displays Edit, Update, and Cancel links in response to changes in the DataGrid's EditItemIndex property.</td>
    </tr>
</table> 
</div>

<!--BoundColumn-->
<p>By explicitly creating <b>Bound Columns</b> in the grid's Columns collection, you can control the order 
and rendering of each column.  This example shows how to create bound columns.<p>
<p>
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid2.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid2.src"
  Icon="/quickstart/aspplus/images/ref_datagrid2.gif"
  Caption="DataGrid2.aspx"
  runat="server" />

<!--HyperLinkColumn-->
<p>A <b>HyperLink Column</b> presents the bound data in HyperLink controls.  This is typically used to navigate 
from an item in the grid to a details view on another page. In this example, the value IntegerValue data field 
is passing as an argument in the URL to another page, and the StringValue data field is used as the display text 
of the hyperlink.
<p>
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid3.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid3.src"
  Icon="/quickstart/aspplus/images/ref_datagrid3.gif"
  Caption="DataGrid3.aspx"
  runat="server" />

<!--ButtonColumn-->
<p>A <b>Button Column</b> is used to bubble a user command from within a row to an event handler on the grid.
In this example, the "Add To Cart" and "Remove From Cart" commands cause the item from the row where
the button was clicked to be added or removed from a simple shopping cart.<p>
<p>
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid4.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid4.src"
  Icon="/quickstart/aspplus/images/ref_datagrid4.gif"
  Caption="DataGrid4.aspx"
  runat="server" />

<!--TemplateColumn-->
<p>With a <b>Template Column</b> you completely control which controls are rendered in the column, and which data fields
are bound to the controls.  This example includes two template columns.  The first column renders two LinkButtons.
These bubble commands to the grid's <b>OnItemCommand</b>, just as a button column does. The last column binds the
true/false value to a read-only CheckBox.<p>
<p>

<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid5.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid5.src"
  Icon="/quickstart/aspplus/images/ref_datagrid5.gif"
  Caption="DataGrid5.aspx"
  runat="server" />

<!--EditCommandColumn-->
<p>The <b>Edit Command Column</b> is a special column type that supports in-place editing of the data in one row in the grid.
EditCommandColumn interacts with another property of the grid: <b>EditItemIndex</b>.  By default the value of
EditItemIndex is -1, meaning none of the rows (items) in the grid is being edited.  If EditItemIndex is -1, an
"edit" button is displayed in the EditCommandColumn for each of the row in the grid.<p>

When the "edit" button is clicked, the grid's OnEditCommand event is thrown.  It's up to you to handle this event
in your code.  The typical logic sets EditItemIndex to the selected row, then rebinds the data to the Grid.<p>

When EditItemIndex is set to a particular row, the EditCommandColumn displays "update" and "cancel" buttons for
that row ("edit" is still displayed for the other rows).  These buttons cause the OnUpdateCommand and OnCancelCommand
event to be thrown, respectively.<p>
<p>
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid6.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid6.src"
  Icon="/quickstart/aspplus/images/ref_datagrid6.gif"
  Caption="DataGrid6.aspx"
  runat="server" />


<!--BEGIN SECTION-->
<br>
<a name="editing">
<span class="subhead">Editing Data in DataGrid</span>
<p>In the preview example, we use the Edit Command Column to support in-place editing of a single row of data.  When 
you use EditItemIndex, the grid automatically inserts the values to be editing into TextBox and CheckBox controls.  

<p>By using template columns for the fields you wish to edit, you can precisely control how the data is edited.  
In this example, the Quantity and Gift Wrap fields are editable in all rows.  When the "Update Totals" button 
is clicked, the grid's Items collection is walked to extract the current values for these fields, and the data
source is updated.

<!--editing data-->
<p>
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid13.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid13.src"
  Icon="/quickstart/aspplus/images/ref_datagrid13.gif"
  Caption="DataGrid13.aspx"
  runat="server" />


<!--BEGIN SECTION-->
<br>
<a name="hiding">
<span class="subhead">Hiding Columns in DataGrid</span>
<p>Each column in the grid has a <b>Visible</b> property.  Setting Visible to false will hide a column.
<!--controlling visibility of a column-->
<p>
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid7.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid7.src"
  Icon="/quickstart/aspplus/images/ref_datagrid7.gif"
  Caption="DataGrid7.aspx"
  runat="server" />

<!--BEGIN SECTION-->
<br>
<a name="sorting">
<span class="subhead">Sorting Columns in DataGrid</span>
<p>Data in a grid is commonly sorted by clicking the header of the column you wish to sort.  You can enable 
sorting in DataGrid by setting <b>AllowSorting</b>=true.  When enabled, the grid will render LinkButtons
in the header for each column.  When the button is clicked, the grid's OnSortCommand event is thrown.  It's up 
to you to handle this event in your code.  Since DataGrid always displays the data in the same order it occurs 
in the datasource, the typical logic sorts the datasource, then rebinds the data to the grid.<p>

This example shows how to implement simple sorting in DataGrid.
<!--sorting 1-->
<p>
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid8.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid8.src"
  Icon="/quickstart/aspplus/images/ref_datagrid8.gif"
  Caption="DataGrid8.aspx"
  runat="server" />

<!--sorting 2-->
<p>You can customize the grid's sorting behavior by creating the Columns collection and manipulating the SortField 
for each column.  (For auto-generated columns, the SortField is the same as the data field.)  Omitting SortField on
a column will disable sorting by that column.<p>

You can also create an Outlook-style sort button by using a template column.  Note the button must return the "sort"
command.<p>
<p>
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid9.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid9.src"
  Icon="/quickstart/aspplus/images/ref_datagrid9.gif"
  Caption="DataGrid9.aspx"
  runat="server" />

<!--BEGIN SECTION-->
<br>
<a name="paging">
<span class="subhead">Paging in DataGrid</span>
<p>The DataGrid provides the means to display a group of records from the datasource (for example, the
first 10), then navigate to the "page" of next 10 records, and so on through the data.

<p>You enable paging in DataGrid by setting <b>AllowPaging</b>=true.  When enabled, the grid will display page 
navigation buttons either as "next/previous" buttons or as numeric buttons.  When a page navigation button is 
clicked, the grid increments its <b>CurrentPageIndex</b> property, then throws the OnPageCommand event.  
It's up to you to handle this event in your code.  The most basic logic simply rebinds the datasource 
as in this example.<p>

<!--Paging 1-->
<p>
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid10.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid10.src"
  Icon="/quickstart/aspplus/images/ref_datagrid10.gif"
  Caption="DataGrid10.aspx"
  runat="server" />

<p>The DataGrid has built-in page navigation controls, called the "pager," which displays either as "next/previous"
buttons or as a set of numeric page buttons.  You can hide the built-in pager UI and supply your own paging UI.<p>

To change the page, your UI must set the <b>CurrentPageIndex</b> to the desired page (base 0), and rebind the
datasource to the grid.<p>

<!--Providing Alternate Paging UI-->
<p>
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid11.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid11.src"
  Icon="/quickstart/aspplus/images/ref_datagrid11.gif"
  Caption="DataGrid11.aspx"
  runat="server" />

<p>For very large data models, it is expensive to load the entire data source each time the grid is paged.  A
common alternative is to retrieve the data in page-size "chunks," rather than retrieving all of the the
the data and then stepping into the current row.<p>

The DataGrid supports chunking via the <b>AllowCustomPaging</b> and <b>VirtualItemCount</b> properties.  If
AllowCustomPaging is true, the DataGrid does not calculate a starting display position in the data model based
on CurrentPageIndex.  Instead, DataGrid displays all of the data in the data model, while its Pager
reports the current position as page CurrentPageIndex of (VirtualItemCount+PageSize-1)/PageSize.<p>
<!--Custom Paging-->
<p>
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid12.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/datagrid/datagrid12.src"
  Icon="/quickstart/aspplus/images/ref_datagrid12.gif"
  Caption="DataGrid12.aspx"
  runat="server" />

<!-- #include virtual="/quickstart/aspplus/include/footer.inc" -->
