
<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/aspplus/include/header.inc" -->

<h4>DataList</h4>

<p>
<div class="indent" style="font-family:Verdana; font-size:8pt;">
    <b> <img align="middle" src="/quickstart/images/bullet.gif">&nbsp;&nbsp;</b>
    <a class="toc2" target="content" href="#working">Working With DataList</a><br>
    <b> <img align="middle" src="/quickstart/images/bullet.gif">&nbsp;&nbsp;</b>
    <a class="toc2" target="content" href="#selecting">Selecting Items in DataList</a><br>
    <b> <img align="middle" src="/quickstart/images/bullet.gif">&nbsp;&nbsp;</b>
    <a class="toc2" target="content" href="#editing">Editing DataList Items</a><br>
</div>

<p>
<hr>

<!--BEGIN SECTION-->
<br>
<a name="working">
<span class="subhead">Working With DataList</span>
<p>The DataList server control displays data items in a repeating list, and optionally supports selecting and 
editing the items.  The content and layout of list items in DataList is defined using <b>templates</b>.  At a 
minimum, every DataList must define an <b>ItemTemplate</b>, however several optional templates may be used to 
customize the appearance of the list:<p>

<div class="indent">
<table class="table2" width="75%">
    <tr>
    <th style="width:25%"><b>Template Name</b></th>
    <th><b>Description</b></th>
    </tr>
    <tr>
    <td style="width:25%"><b>ItemTemplate</b></td>
    <td>Defines the content and layout of items within the list.  <b>Required.</b></td>
    </tr>
    <tr>
    <td style="width:25%"><b>AlternatingItemTemplate</b></td>
    <td>If defined, this determines the content and layout of alternating items.  If not defined, ItemTemplate is used.</td>
    </tr>
    <tr>
    <td style="width:25%"><b>SeparatorTemplate</b></td>
    <td>If defined, this is rendered between items (and alternating items).  If not defined, a separator is not rendered.</td>
    </tr>
    <tr>
    <td style="width:25%"><b>SelectedItemTemplate</b></td>
    <td>If defined, this determines the content and layout of the selected item.  If not defined, ItemTemplate is used.</td>
    </tr>
    <tr>
    <td style="width:25%"><b>EditItemTemplate</b></td>
    <td>If defined, this determines the content and layout of the item being edited.  If not defined, ItemTemplate is used.</td>
    </tr>
    <tr>
    <td style="width:25%"><b>HeaderTemplate</b></td>
    <td>If defined, this determines the content and layout of the list header.  If not defined, header is not rendered.</td>
    </tr>
    <tr>
    <td style="width:25%"><b>FooterTemplate</b></td>
    <td>If defined, this determines the content and layout of the list footer.  If not defined, footer is not rendered.</td>
    </tr>
</table> 
</div>

<p>Templates define the HTML elements and controls that should be displayed for an item, as well as the layout of these
elements.  Style formatting -- font, color and border attributes -- is set via <b>styles</b>.  Each template has its 
own style property.  For example, styles for the EditItemTemplate are set via the <b>EditItemStyle</b> property.

<p>A third set of properties affect the overall rendering of DataList.  By default, DataList items render within a
table as a single vertical column.  Setting the <b>RepeatLayout</b> property to Flow will remove the HTML table 
structure from the rendering of the list.

<p>DataList supports directional rendering via the <b>RepeatDirection</b> property, 
meaning it can render its items horizontally or vertically.  Since page width is the dimension that the developer 
must control in Web UI, DataList permits the developer to control the number of "columns" that are rendered 
(<b>RepeatColumns</b>), regardless of whether the items are rendered horizontally or vertically.  

<p>If RepeatDirection is Horizontal and RepeatColumns is five, items will be rendered in a "rows" of 5, until 
all items are rendered:<p>

<div class="indent">
<table class="code" width="400">
<tr>
    <td width="20%">1</td>
    <td width="20%">2</td>
    <td width="20%">3</td>
    <td width="20%">4</td>
    <td width="20%">5</td>
</tr><tr>
    <td>6</td>
    <td>7</td>
    <td>8</td>
    <td>9</td>
    <td>10</td>
</tr><tr>
    <td>11</td>
    <td>12</td>
    <td>13</td>
    <td>&nbsp;</td>
    <td>&nbsp;</td>
</tr>
</table>
</div>

<p>Conversely, if RepeatDirection is Vertical, items are rendered in 5 "columns", each equal in length to the 
total number of items divided by 5:<p>

<div class="indent">
<table class="code" width="400">
<tr>
    <td width="20%">1</td>
    <td width="20%">4</td>
    <td width="20%">7</td>
    <td width="20%">10</td>
    <td width="20%">13</td>
</tr><tr>
    <td>2</td>
    <td>5</td>
    <td>8</td>
    <td>11</td>
    <td>&nbsp;</td>
</tr><tr>
    <td>3</td>
    <td>6</td>
    <td>9</td>
    <td>12</td>
    <td>&nbsp;</td>
</tr>
</table>
</div>


<p>This sample illustrates using a simple DataList control.

<p>
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/DataList/DataList1.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/DataList/DataList1.src"
  Icon="/quickstart/aspplus/images/ref_datalist1.gif"
  Caption="DataList1.aspx"
  runat="server" />

<!--BEGIN SECTION-->
<p>
<a name="selecting">
<span class="subhead">Selecting Items in DataList</span>
<p>You can customize the content and appearance of the selected item via the <b>SelectedItemTemplate</b>
property.  The SelectedItemTemplate is controlled by the <b>SelectedIndex</b> property.  By default the value of
SelectedIndex is -1, meaning none of the items in the list is selected.  When SelectedIndex is set to a 
particular item, that item is displayed using the SelectedItemTemplate.<p>

<p>This sample illustrates using a SelectedItemTemplate in DataList.

<p>
<!--selected item-->
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/DataList/DataList2.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/DataList/DataList2.src"
  Icon="/quickstart/aspplus/images/ref_datalist2.gif"
  Caption="DataList2.aspx"
  runat="server" />

<!--BEGIN SECTION-->
<p>
<a name="editing">
<span class="subhead">Editing DataList Items</span>
<p>The DataList control supports in-place editing of the data in an item via its <b>EditItemTemplate</b>
property.  The EditItemTemplate defines the content and appearance of the item when it is being edited.  For
example, this template includes a textbox, an "Update" button and a "Cancel button":<p>

<div class="code">
  &lt;template name="EditItemTemplate"&gt;<br>
    Item: &lt;asp:TextBox id="Text1" runat="server"<br>
    &nbsp;&nbsp;Text='&lt;%# ((DataRowView)Container.DataItem)["Item"]  %&gt;'<br>
    &nbsp;&nbsp;/>&lt;br&gt;<br>
    &lt;asp:LinkButton id="button1" runat="server"<br>
    &nbsp;&nbsp;Text="Update"<br>
    &nbsp;&nbsp;Command="update"<br>
    &nbsp;&nbsp;/&gt;<br>
    &lt;asp:LinkButton id="button2" runat="server"<br>
    &nbsp;&nbsp;Text="Cancel"<br>
    &nbsp;&nbsp;Command="cancel"<br>
    &nbsp;&nbsp;/&gt;<br>
  &lt;/template&gt;<br>
</div>

<p>The EditItemTemplate interacts with another property: <b>EditItemIndex</b>.  By default the value of
EditItemIndex is -1, meaning none of the items in the list is being edited.  When EditItemIndex is set to a 
particular item, that item is displayed using the EditItemTemplate.<p>

<p>The DataList also supplies three events that may be used to support editing.  <b>OnEditCommand</b> is thrown 
when an "edit" command button control is clicked within the list's ItemTemplate.  It's up to you to handle this event
in your code.  The typical logic sets EditItemIndex to the selected item, then rebinds the data to the DataList:

<div class="code"><xmp>
    protected void DataList_EditCommand(object Source, DataListCommandEventArgs e)
    {
        DataList1.EditItemIndex = (int)e.Item.ItemIndex;
        BindList();
    }
</xmp></div>

The EditItemTemplate will typically contain "update" and "cancel" command buttons.  These buttons cause the 
<b>OnUpdateCommand</b> and <b>OnCancelCommand</b> events to be thrown, respectively.  It's up to you to handle this event
in your code.  The typical logic for "cancel" sets EditItemIndex to -1, then rebinds the data to the DataList:<p>

<div class="code"><xmp>
    protected void DataList_CancelCommand(object Source, DataListCommandEventArgs e)
    {
        DataList1.EditItemIndex = -1;
        BindList();
    }
</xmp></div>

<p>The typical logic for "update" updates the data source, then sets EditItemIndex to -1 and rebinds the data to the 
DataList.  This sample illustrates editing items in DataList.

<p>
<!--edit item-->
<Acme:SourceRef
  RunSample="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/DataList/DataList3.aspx"
  ViewSource="/quickstart/aspplus/samples/webforms/ctrlref/webctrl/DataList/DataList3.src"
  Icon="/quickstart/aspplus/images/ref_datalist3.gif"
  Caption="DataList3.aspx"
  runat="server" />

<!-- #include virtual="/quickstart/aspplus/include/footer.inc" -->
